@@ -4,6 +4,7 @@
  * Copyright (C) 2006 Ray Strode <rstrode@redhat.com>
  * Copyright (C) 2003 Bill Nottingham <notting@redhat.com>
  * Copyright (c) 1993-2003 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -185,8 +186,9 @@ auth_message_handler (GSAuthMessageStyle style,
 }
 
 static gboolean
-gs_auth_queued_message_handler (GsAuthMessageHandlerData *data)
+gs_auth_queued_message_handler (gpointer user_data)
 {
+	GsAuthMessageHandlerData *data = user_data;
 	gboolean res;
 
 	if (gs_auth_get_verbose ())
@@ -237,7 +239,7 @@ gs_auth_run_message_handler (struct pam_closure *c,
 
 	/* Queue the callback in the gui (the main) thread
 	 */
-	g_idle_add ((GSourceFunc) gs_auth_queued_message_handler, &data);
+	g_idle_add (gs_auth_queued_message_handler, &data);
 
 	if (gs_auth_get_verbose ())
 	{
@@ -509,9 +511,10 @@ set_pam_error (GError **error,
 
 }
 
-static int
-gs_auth_thread_func (int auth_operation_fd)
+static gpointer
+gs_auth_thread_func (gpointer data)
 {
+	int              auth_operation_fd;
 	static const int flags = 0;
 	int              status;
 	int              status2;
@@ -519,6 +522,8 @@ gs_auth_thread_func (int auth_operation_fd)
 	sigset_t         set;
 	const void      *p;
 
+	auth_operation_fd = GPOINTER_TO_INT (data);
+
 	timeout.tv_sec = 0;
 	timeout.tv_nsec = 1;
 
@@ -602,7 +607,7 @@ done:
 	 */
 	close (auth_operation_fd);
 
-	return status;
+	return GINT_TO_POINTER (status);
 }
 
 static gboolean
@@ -664,7 +669,7 @@ gs_auth_pam_verify_user (pam_handle_t *handle,
 	                           (GIOFunc) gs_auth_loop_quit, &thread_done);
 
 	auth_thread = g_thread_new ("auththread",
-                                    (GThreadFunc) gs_auth_thread_func,
+                                    gs_auth_thread_func,
                                     GINT_TO_POINTER (auth_operation_fds[1]));
 
 	if (auth_thread == NULL)
@@ -694,6 +699,7 @@ out:
 	if (watch_id != 0)
 	{
 		g_source_remove (watch_id);
+		watch_id = 0;
 	}
 
 	if (channel != NULL)
